local assets =
{
    Asset("ANIM", "anim/goddess_deer_build.zip"),
    Asset("ANIM", "anim/deer_basic.zip"),
    Asset("ANIM", "anim/goddess_deer_action.zip"),
}

local unshackle_assets =
{
    Asset("ANIM", "anim/deer_build.zip"),
    Asset("ANIM", "anim/deer_unshackle.zip"),
}

local prefabs =
{
    "meat",
    "boneshard",
    "deer_antler",
    "deer_growantler_fx",
}

local redprefabs =
{
    "redgem",
    "meat",
    "deer_fire_circle",
    "deer_fire_charge",
    "deer_unshackle_fx",
}

local blueprefabs =
{
    "bluegem",
    "meat",
    "deer_ice_circle",
    "deer_ice_charge",
    "deer_unshackle_fx",
}

local brain = require("brains/goddessdeerbrain")

SetSharedLootTable('goddess_deer',
{
    {'meat',              0.75},
	{'smallmeat',         1.00},
	{'smallmeat',         0.50},
    {'peach',             0.50},
	{'goddess_ribbon',    0.12},
})

local function KeepTargetFn(inst, target)
    return inst.components.combat:CanTarget(target)
end

local function ShareTargetFn(dude)
    return dude:HasTag("goddess_deer") and not dude.components.health:IsDead()
end

local function OnAttacked(inst, data)
    inst.components.combat:SetTarget(data.attacker)
    inst.components.combat:ShareTarget(data.attacker, 20, ShareTargetFn, 3)
end

local function DoNothing()
end

local function DoChainIdleSound(inst, volume)
    inst.SoundEmitter:PlaySound("dontstarve/creatures/together/deer/chain_idle", nil, volume)
end

local function DoBellIdleSound(inst, volume)
    inst.SoundEmitter:PlaySound("dontstarve/creatures/together/deer/bell_idle", nil, volume)
end

local function DoChainSound(inst, volume)
    inst.SoundEmitter:PlaySound("dontstarve/creatures/together/deer/chain", nil, volume)
end

local function DoBellSound(inst, volume)
    inst.SoundEmitter:PlaySound("dontstarve/creatures/together/deer/bell", nil, volume)
end

local function NormalRetargetFn(inst)
    if inst.components.follower:GetLeader() ~= nil and inst.components.combat:TargetIs() == nil then
		local leader = inst.components.follower:GetLeader()
		inst.components.combat:SetTarget(leader.components.combat:TargetIs())
	end
end

local function SetupSounds(inst)
    if inst.gem == nil then
        inst.DoChainSound = DoNothing
        inst.DoChainIdleSound = DoNothing
        inst.DoBellSound = DoNothing
        inst.DoBellIdleSound = DoNothing
    elseif IsSpecialEventActive(SPECIAL_EVENTS.WINTERS_FEAST) then
        inst.DoChainSound = DoChainSound
        inst.DoChainIdleSound = DoChainIdleSound
        inst.DoBellSound = DoBellSound
        inst.DoBellIdleSound = DoBellIdleSound
    else
        inst.DoChainSound = DoChainSound
        inst.DoChainIdleSound = DoChainIdleSound
        inst.DoBellSound = DoNothing
        inst.DoBellIdleSound = DoNothing
    end
end

local function ShouldAcceptItem(inst, item)	
	if item:HasTag("peachy") then
            return inst.components.follower.leader == nil or inst.components.follower:GetLoyaltyPercent() <= 0.9
	else
		local last_eat_time = inst.components.eater:TimeSinceLastEating()
            return (last_eat_time == nil or
                    last_eat_time >= 180)
	end
end

local function OnGetItemFromPlayer(inst, giver, item)
	if 	inst.components.eater:CanEat(item) and item:HasTag("peachy") then
			giver:PushEvent("makefriend")
			giver.components.leader:AddFollower(inst)
			inst.components.follower:AddLoyaltyTime(item.components.edible:GetHunger() * TUNING.PIG_LOYALTY_PER_HUNGER)
			inst.components.eater:Eat(item)
			inst.components.follower.maxfollowtime =
				giver:HasTag("polite")
				and TUNING.PIG_LOYALTY_MAXTIME + TUNING.PIG_LOYALTY_POLITENESS_MAXTIME_BONUS
				or TUNING.PIG_LOYALTY_MAXTIME
	else
		inst.components.eater:Eat(item)
		inst:DoTaskInTime(3, SpawnPrefab("poop").Transform:SetPosition(inst.Transform:GetWorldPosition()))
    end
end
	
local function OnEat(inst, food)
    if food.components.edible then
	end
end

local function common_fn(gem)
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddDynamicShadow()
    inst.entity:AddNetwork()

    inst.DynamicShadow:SetSize(1.75, .75)

    inst.Transform:SetSixFaced()

    MakeCharacterPhysics(inst, 100, .5)

    inst.AnimState:SetBank("goddess_deer")
    inst.AnimState:SetBuild("goddess_deer_build")
    inst.AnimState:PlayAnimation("idle_loop", true)
	
	inst.AnimState:OverrideSymbol("swap_neck_collar", "goddess_deer_build", "swap_neck")
	
	inst.AnimState:Hide("CHAIN")
	
	local s = 0.7 + (math.random()/2.5)
	inst.Transform:SetScale(s,s,s)

	inst:AddTag("saltlicker")
    inst:AddTag("deer")
    inst:AddTag("animal")
	inst:AddTag("goddess_deer")
	inst:AddTag("follower")
	inst:AddTag("companion")
	inst:AddTag("healthinfo")

    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end

    inst:AddComponent("timer")
    inst:AddComponent("knownlocations")
	
    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(TUNING.DEER_GEMMED_HEALTH)

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(TUNING.DEER_GEMMED_DAMAGE)
    inst.components.combat.hiteffectsymbol = "deer_torso"
    inst.components.combat:SetRange(TUNING.DEER_ATTACK_RANGE)
    inst.components.combat:SetAttackPeriod(TUNING.DEER_ATTACK_PERIOD)
    inst.components.combat:SetKeepTargetFunction(KeepTargetFn)
	inst.components.combat:SetRetargetFunction(3, NormalRetargetFn)
    inst.components.combat:SetTarget(nil)
    inst.components.combat:SetHurtSound("dontstarve/creatures/together/deer/hit")
    inst:ListenForEvent("attacked", OnAttacked)

    inst:AddComponent("sleeper")
    inst.components.sleeper:SetResistance(4)

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetChanceLootTable('goddess_deer')

    inst:AddComponent("inspectable")

    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = TUNING.DEER_WALK_SPEED
    inst.components.locomotor.runspeed = TUNING.DEER_RUN_SPEED

    MakeHauntablePanic(inst)

    SetupSounds(inst)
    inst:SetStateGraph("SGdeer")

    inst:AddComponent("saltlicker")
    inst.components.saltlicker:SetUp(TUNING.SALTLICK_DEER_USES)
	
	inst:AddComponent("follower")
	inst.components.follower.maxfollowtime = TUNING.PIG_LOYALTY_MAXTIME
	
	inst:AddComponent("trader")
	inst.components.trader:SetAcceptTest(ShouldAcceptItem)
    inst.components.trader.onaccept = OnGetItemFromPlayer
	
	inst:AddComponent("eater")
    inst.components.eater:SetDiet({ FOODGROUP.OMNI }, { FOODGROUP.OMNI })
--	inst.components.eater:SetOnEatFn(OnEat)
	
	inst:AddComponent("brushable")
    inst.components.brushable.regrowthdays = 2
    inst.components.brushable.max = 1
    inst.components.brushable.prize = "goddess_ribbon"

    inst:SetBrain(brain)

	return inst
end

local function fn()
    return common_fn()
end

return 	Prefab("goddess_deer", fn, assets, prefabs)