local assets =
{
    Asset("ANIM", "anim/goddess_flute.zip"),
}

local function ruinshat_unproc(inst)
	if inst:HasTag("forcefield") then
		inst:RemoveTag("forcefield")
		if inst._fx ~= nil then
			inst._fx:kill_fx()
			inst._fx = nil
		end
		inst.components.armor:SetAbsorption(TUNING.ARMOR_RUINSHAT_ABSORPTION)
		inst.components.armor.ontakedamage = nil
	end
end

local function HearPanFlute(inst, musician, instrument)
	
	if inst ~= musician and
        (TheNet:GetPVPEnabled() or not inst:HasTag("player")) and
        not (inst.components.freezable ~= nil and inst.components.freezable:IsFrozen()) and
        not (inst.components.pinnable ~= nil and inst.components.pinnable:IsStuck()) then
			if inst.components.sleeper ~= nil then
				inst.components.sleeper:AddSleepiness(10, TUNING.PANFLUTE_SLEEPTIME*2)
			elseif inst.components.grogginess ~= nil then
				inst.components.grogginess:AddGrogginess(10, TUNING.PANFLUTE_SLEEPTIME*2)
			else
				inst:PushEvent("knockedout")
			end
    end
	
	if musician:HasTag("windy1") and musician:HasTag("windy2") then
		local pos = inst:GetPosition()
		local ents = TheSim:FindEntities(pos.x, pos.y, pos.z, TUNING.PANFLUTE_SLEEPRANGE, nil, {"FX", "NOCLICK","DECOR","INLIMBO"}, {"companion", "follower", "player"})
		for i,v in pairs(ents) do
			if v.components.health then
				v.components.health:DoDelta(75)
			end
			if v.components.sanity then
				v.components.sanity:DoDelta(100)
			end
		end
	end
	
end

local function ShouldAcceptItem(inst, item)
    return item:HasTag("magicpowder")
end

local function OnGetItem(inst, giver, item)
	local finiteuses =  inst.components.finiteuses:GetPercent()
	if giver:HasTag("windy1") and giver:HasTag("windy2") then
		inst.components.finiteuses:SetPercent(1)
	else
		inst.components.finiteuses:SetPercent(finiteuses + 0.10)
	end
	if finiteuses >= 1 then
		inst.components.finiteuses:SetPercent(1)
	end
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddNetwork()

    MakeInventoryPhysics(inst)

    inst:AddTag("flute")

    inst.AnimState:SetBank("pan_flute")
    inst.AnimState:SetBuild("goddess_flute")
    inst.AnimState:PlayAnimation("idle")

    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end

    inst:AddComponent("inspectable")
    inst:AddComponent("instrument")
    inst.components.instrument.range = TUNING.PANFLUTE_SLEEPRANGE
    inst.components.instrument:SetOnHeardFn(HearPanFlute)

    inst:AddComponent("tool")
    inst.components.tool:SetAction(ACTIONS.PLAY)

    inst:AddComponent("finiteuses")
    inst.components.finiteuses:SetMaxUses(TUNING.PANFLUTE_USES)
    inst.components.finiteuses:SetUses(TUNING.PANFLUTE_USES)
    inst.components.finiteuses:SetOnFinished(inst.Remove)
    inst.components.finiteuses:SetConsumption(ACTIONS.PLAY, 1)
	
	inst:AddComponent("trader")
	inst.components.trader:SetAcceptTest(ShouldAcceptItem)
    inst.components.trader.onaccept = OnGetItem
	
    inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/inventoryimages/goddess_flute.xml"

    MakeHauntableLaunch(inst)
    AddHauntableCustomReaction(inst, function(inst, haunter)
        if math.random() <= TUNING.HAUNT_CHANCE_HALF then
            if inst.components.finiteuses then
                inst.components.finiteuses:Use(1)
                inst.components.hauntable.hauntvalue = TUNING.HAUNT_MEDIUM
                return true
            end
        end
        return false
    end, true, false, true)

    return inst
end

return Prefab("goddess_flute", fn, assets)