local assets =
{
    Asset("ANIM", "anim/goddess_lantern.zip"),
	Asset("IMAGE", "images/inventoryimages/goddess_lantern.tex"),
	Asset("ATLAS", "images/inventoryimages/goddess_lantern.xml"),
	Asset("ANIM", "anim/swap_goddess_lantern.zip"),
	Asset("ANIM", "anim/swap_goddess_lantern1.zip"),
}

local function onequip(inst, owner)
	owner:AddTag("windy3")
	owner.SoundEmitter:PlaySound("dontstarve/wilson/equip_item_gold")
	if inst.on == true then
		owner.AnimState:OverrideSymbol("swap_object", "swap_goddess_lantern", "swap_goddess_lantern")
		owner.AnimState:Show("ARM_carry")
		owner.AnimState:Hide("ARM_normal")
		if inst.components.fueled ~= nil then
			inst.components.fueled:StartConsuming()
		end
		if inst.fire == nil then
			inst.fire = SpawnPrefab("goddess_lantern_fire")
			inst.fire.entity:AddFollower()
			inst.fire.Follower:FollowSymbol(owner.GUID, "swap_object", 0, -110, 0)
		end
	elseif inst.on == false then
		owner.AnimState:OverrideSymbol("swap_object", "swap_goddess_lantern1", "swap_goddess_lantern1")
		owner.AnimState:Show("ARM_carry")
		owner.AnimState:Hide("ARM_normal")
	end
end

local function onunequip(inst, owner)
    owner.AnimState:Hide("ARM_carry")
    owner.AnimState:Show("ARM_normal")
	owner:RemoveTag("windy3")
	if inst.components.fueled ~= nil then
        inst.components.fueled:StopConsuming()
    end
	if inst.fire ~= nil then
        inst.fire:Remove()
        inst.fire = nil
    end
end

local function TurnOff(inst, instant)
    inst.on = false
	inst.Light:Enable(false)
	inst.AnimState:PlayAnimation("idle", true)
	inst.SoundEmitter:PlaySound("dontstarve/creatures/together/hutch/light_off")
	if inst.components.fueled ~= nil then
        inst.components.fueled:StopConsuming()
    end
end

local function TurnOn(inst, instant)
    inst.on = true
	inst.Light:Enable(true)
	inst.AnimState:PlayAnimation("lit_pre", true)
	inst.AnimState:PushAnimation("idle_lit")
	inst.SoundEmitter:PlaySound("dontstarve/creatures/together/hutch/light_on")
	if inst.components.fueled ~= nil then
        inst.components.fueled:StartConsuming()
    end
end

local function ShouldAcceptItem(inst, item)
    return item:HasTag("magicpowder")
end

local function OnGetItem(inst, giver, item)
	local fueled =  inst.components.fueled:GetPercent()
	if giver:HasTag("windy1") and giver:HasTag("windy2") then
		inst.components.fueled:SetPercent(1)
	else
		inst.components.fueled:SetPercent(fueled + 0.10)
	end
	if fueled >= 1 then
		inst.components.fueled:SetPercent(1)
	end
end

local function toground(inst)
	if inst.on == (false) then
		inst.Light:Enable(false)
	elseif inst.on == (true) then
		inst.Light:Enable(true)
	end
end

local function unslow(inst)
	inst.components.locomotor.externalspeedmultiplier = 1
	inst:RemoveTag("slowed")
end

local function getspawnlocation(inst, target)
    local x1, y1, z1 = inst.Transform:GetWorldPosition()
    local x2, y2, z2 = target.Transform:GetWorldPosition()
    return x1 + .15 * (x2 - x1), 0, z1 + .15 * (z2 - z1)
end

local function trygrowth(inst)
    if inst:IsInLimbo()
        or (inst.components.witherable ~= nil and
            inst.components.witherable:IsWithered()) then
        return
    end
    if inst.components.pickable ~= nil then
        if inst.components.pickable:CanBePicked() and inst.components.pickable.caninteractwith then
            return
        end
        inst.components.pickable:FinishGrowing()
    end
    if inst.components.crop ~= nil then
        inst.components.crop:DoGrow(TUNING.TOTAL_DAY_TIME * 3, true)
    end
    if inst.components.growable ~= nil and
        (inst:HasTag("tree") or inst:HasTag("winter_tree")) and
        not inst:HasTag("stump") then
        inst.components.growable:DoGrowth()
    end
    if inst.components.harvestable ~= nil and inst.components.harvestable:CanBeHarvested() and inst:HasTag("mushroom_farm") then
        inst.components.harvestable:Grow()
    end
end

local function spells(staff, target)
	staff.SoundEmitter:PlaySound("dontstarve/creatures/together/deer/bell")
	local fueled =  staff.components.fueled:GetPercent()
	
	if fueled > 0 then
		if target:IsInLimbo()
			or (target.components.witherable ~= nil and
				target.components.witherable:IsWithered()) then
			return
		end
	-- Plant/crop/mushroom farm growth spell
		if target.components.pickable ~= nil or target.components.crop ~= nil or target.components.growable ~= nil or target.components.harvestable ~= nil then
			local pos = staff:GetPosition()
            local range = 20
            local ents = TheSim:FindEntities(pos.x, pos.y, pos.z, range, nil, { "pickable", "stump", "withered", "INLIMBO" })
            
			if #ents > 0 then
                trygrowth(table.remove(ents, math.random(#ents)))
                if #ents > 0 then
                    local timevar = 1 - 1 / (#ents + 1)
                    for i, v in ipairs(ents) do
                        v:DoTaskInTime(timevar * math.random(), trygrowth)
                    end
                end
				SpawnPrefab("firering_fx").Transform:SetPosition(getspawnlocation(staff, target))
				local fueled =  staff.components.fueled:GetPercent()
				staff.components.fueled:SetPercent(fueled - 0.19)
				if fueled <=0 then
					staff:Remove()
				end
				
            end	
		end
	-- Passify spell
		if target.components.combat then
			if target.components.combat:HasTarget() then
				target.components.combat:DropTarget()
			end
		end
	-- Slow spell
		if not target:HasTag("slowed") and not target:HasTag("goddess_deer") then 
			if not target.components.follower then
				if target.components.locomotor ~= nil and not target:HasTag("epic") then
					target.components.locomotor.externalspeedmultiplier = 0.35
					target:AddTag("slowed")
					target:DoTaskInTime(10, unslow)
				elseif target.components.locomotor ~= nil and target:HasTag("epic") then
					target.components.locomotor.externalspeedmultiplier = 0.60
					target:AddTag("slowed")
					target:DoTaskInTime(5, unslow)
				end
			elseif target.components.follower then
				local leader = target.components.follower:GetLeader()
				if leader == nil then
					if target.components.locomotor ~= nil and not target:HasTag("epic") then
						target.components.locomotor.externalspeedmultiplier = 0.35
						target:AddTag("slowed")
						target:DoTaskInTime(10, unslow)
					elseif target.components.locomotor ~= nil and target:HasTag("epic") then
						target.components.locomotor.externalspeedmultiplier = 0.60
						target:AddTag("slowed")
						target:DoTaskInTime(5, unslow)
					end
				end
			end
		end
	-- Deer befriending spell
		if target.components.follower and target:HasTag("goddess_deer") then
			target:DoTaskInTime(0.1, function (inst)
				local pos = inst:GetPosition()
				local ents = TheSim:FindEntities(pos.x, pos.y, pos.z, 30, nil, {"FX", "NOCLICK","DECOR","INLIMBO"}, {"player", "windy3"})
				for i,v in pairs(ents) do
					if v.components.leader then 
						v:PushEvent("makefriend")
						v.components.leader:AddFollower(inst)
						inst.components.follower:AddLoyaltyTime(60)
						inst.components.health:DoDelta(160)
					end
				end
			end)
		end
		
		local fueled =  staff.components.fueled:GetPercent()
		staff.components.fueled:SetPercent(fueled - 0.01)
		if fueled <=0 then
			staff:Remove()
		end
		
	end
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddNetwork()
	inst.entity:AddLight()
	
	
	inst.Light:Enable(false)
    inst.Light:SetIntensity(0.75)
    inst.Light:SetColour(185 / 255, 255 / 255, 150 / 255)
    inst.Light:SetFalloff(.9)
    inst.Light:SetRadius(8)
	
	local s = 1.5
	inst.Transform:SetScale(s,s,s)

    MakeInventoryPhysics(inst)

    inst.AnimState:SetBank("goddess_lantern")
    inst.AnimState:SetBuild("goddess_lantern")
    inst.AnimState:PlayAnimation("idle", true)

	inst:AddTag("nopunch")
    inst:AddTag("quickcast")
	
    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end
	
    inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/inventoryimages/goddess_lantern.xml"
	
	inst:AddComponent("equippable")
	inst.components.equippable:SetOnEquip(onequip)
	inst.components.equippable:SetOnUnequip(onunequip)
	
	inst:AddComponent("fueled")
    inst.components.fueled:InitializeFuelLevel(480*3)
    inst.components.fueled:SetFirstPeriod(TUNING.TURNON_FUELED_CONSUMPTION, TUNING.TURNON_FULL_FUELED_CONSUMPTION)
	inst.components.fueled:SetDepletedFn(inst.Remove)
	
	inst:AddComponent("trader")
	inst.components.trader:SetAcceptTest(ShouldAcceptItem)
    inst.components.trader.onaccept = OnGetItem
	
	inst:AddComponent("machine")
    inst.components.machine.turnonfn = TurnOn
    inst.components.machine.turnofffn = TurnOff
    inst.components.machine.cooldowntime = 0.5
	
	inst:AddComponent("spellcaster")
    inst.components.spellcaster.canuseontargets = true
    inst.components.spellcaster.canonlyuseonworkable = true
	inst.components.spellcaster.canonlyuseoncombat = true
    inst.components.spellcaster.canonlyuseonrecipes = true
    inst.components.spellcaster.quickcast = true
    inst.components.spellcaster:SetSpellFn(spells)
    inst.components.spellcaster.castingstate = "castspell_tornado"
	
	inst:ListenForEvent("ondropped", toground)
	
	inst.on = false

    return inst
end

return Prefab("goddess_lantern", fn, assets)