local assets =
{
    Asset("ANIM", "anim/goddess_shrine.zip"),
	Asset("IMAGE", "images/map_icons/goddess_shrine.tex"),
	Asset("ATLAS", "images/map_icons/goddess_shrine.xml")
}

local GODDESS = {SCIENCE = 0, ANCIENT = 0, GODDESS = 2, MAGIC = 0, SHADOW = 0, CARTOGRAPHY = 0}

local function light(inst)    
	inst.Light:Enable(true)
end

local function extinguish(inst)
	inst.Light:Enable(false)
	inst.AnimState:PlayAnimation("idle")
end

local function complete_onturnon(inst)

	inst.AnimState:PlayAnimation("active")
	inst.AnimState:PushAnimation("active")
    if not inst.SoundEmitter:PlayingSound("idlesound") then
        inst.SoundEmitter:PlaySound("dontstarve/common/ancienttable_LP", "idlesound")
    end
	
end

local function complete_onturnoff(inst)
	inst.AnimState:PlayAnimation("idle")
        inst.SoundEmitter:KillSound("dontstarve/common/ancienttable_LP", "idlesound")
    end

local function complete_doonact(inst)
    if inst._activecount > 1 then
        inst._activecount = inst._activecount - 1
    else
        inst._activecount = 0
        inst.SoundEmitter:KillSound("sound")
    end
	inst.AnimState:PushAnimation("active")
    inst.SoundEmitter:PlaySound("dontstarve/creatures/chester/raise")
end

local function complete_onactivate(inst)
	inst.AnimState:PlayAnimation("use")

    inst._activecount = inst._activecount + 1

    if not inst.SoundEmitter:PlayingSound("sound") then
        inst.SoundEmitter:PlaySound("dontstarve/common/ancienttable_craft", "sound")
    end

    inst:DoTaskInTime(1.5, complete_doonact)
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddNetwork()
	inst.entity:AddMiniMapEntity()
	inst.entity:AddLight()
	
	inst.MiniMapEntity:SetIcon( "goddess_shrine.tex" )
	
	MakeObstaclePhysics(inst, 2)
	inst.Physics:SetCollisionGroup(COLLISION.CHARACTERS)
	inst.Physics:CollidesWith(COLLISION.CHARACTERS)
	
	local s = 4
	inst.Transform:SetScale(s,s,s)

    inst.AnimState:SetBank("goddess_shrine")
    inst.AnimState:SetBuild("goddess_shrine")
    inst.AnimState:PlayAnimation("idle")
	
	inst.Light:Enable(false)
    inst.Light:SetRadius(4)
    inst.Light:SetFalloff(1)
    inst.Light:SetIntensity(0.75)
    inst.Light:SetColour(190 / 255, 250 / 255, 190 / 255)

    inst.entity:SetPristine()

	inst:AddTag("altar")
	inst:AddTag("prototyper")
	inst:AddTag("giftmachine")
	
    if not TheWorld.ismastersim then
        return inst
    end
	
	inst:AddTag("goddess")
	inst:AddTag("antlion_sinkhole_blocker")
	
	inst:AddComponent("prototyper")
	inst.components.prototyper.trees = GODDESS
	
	inst.components.prototyper.onturnon = complete_onturnon
    inst.components.prototyper.onturnoff = complete_onturnoff
    inst.components.prototyper.onactivate = complete_onactivate
	
	inst:AddComponent("playerprox")
    inst.components.playerprox:SetDist(10, 10.1)
    inst.components.playerprox:SetOnPlayerNear(light)
    inst.components.playerprox:SetOnPlayerFar(extinguish)
	
    inst:AddComponent("inspectable")
	
	inst._activecount = 0

    return inst
end

return Prefab("goddess_shrine", fn, assets)